import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import javax.swing.JFrame;
import javax.swing.Timer;

/**
 * Example BetterBallDropper: a program that drops multiple ball
 * objects
 *
 * Jim Teresco, Siena College, Computer Science 225, Spring 2017
 *
 * $Id: templateapp.java 2975 2016-08-30 02:35:29Z terescoj $
 */

public class BallDropper extends JFrame implements MouseListener {
    
    private static final int WINDOW_SIZE = 400;
    private static final int TIMER_DELAY = 30;
    
    /** list of falling balls */
    private ArrayList<FallingBall> balls = new ArrayList<>();

    /**
     * construct a BallDropper frame
     */
    public BallDropper() {
        
        // set window properties
        setSize(WINDOW_SIZE, WINDOW_SIZE);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setVisible(true); 
        
        addMouseListener(this);
        // create and start the timer that will repaint periodically
        Timer t = new Timer(TIMER_DELAY, 
            new ActionListener() {

                // the timer will wake up and call this method every
                // TIMER_DELAY milliseconds.
                @Override
                public void actionPerformed(ActionEvent e) {
                   
                    repaint();
                }
        });
        t.start();
        
    }
    
    /**
     * paint contents of our frame
     * 
     * @param g Graphics object on which to draw
     */
    @Override
    public void paint(Graphics g) {
        
        g.setColor(Color.white);
        g.fillRect(0, 0, WINDOW_SIZE, WINDOW_SIZE);
        g.setColor(Color.black);
        for (FallingBall fb : balls) {
            fb.paint(g);
        }
       
    }
    
    /**
     * mouse listener to create a new ball on each mouse press
     * 
     * @param e mouse event information
     */
    @Override
    public void mousePressed(MouseEvent e) {
     
        balls.add(new FallingBall(e.getX(), e.getY(), WINDOW_SIZE));
        
    }
    
    // remaining methods for MouseListener
    public void mouseReleased(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {}    
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
        
    /**
     * simple main method to allow this class to be run as an application
     * 
     * @param args not used
     */
    public static void main(String[] args) {
	
        new BallDropper();
    }
}
