import java.awt.Color;
import java.awt.Graphics;
import java.util.Random;

/**
 * A Falling Ball that manages its own animation
 * 
 * @author Jim Teresco
 */
public class FallingBall extends Thread {

    private static final int BALL_SIZE = 50;
    //private static final int VELOCITY = 2;
    private static final long PAUSE_TIME = 15;
    
    private static Random r = new Random();
    
    /** current ball position (upper left corner of bounding box) */
    private double x, y;
    /** how far the ball can fall */
    private int yMax;
    
    /** color of this ball */
    private Color c;
    
    /** x "side to side" velocity */
    private double xSpeed;
    
    /** y velocity */
    private double ySpeed = 1.0;
    
    /**
     * construct a FallingBall and start its animation
     * 
     * @param x x-coordinate of the center of the ball
     * @param y y-coordinate of the top of the ball
     * @param yMax the y-coordinate of the bottom of the window
     */
    public FallingBall(int x, int y, int yMax) {
        
        // set initial position of this ball
        this.x = x - BALL_SIZE/2;
        this.y = y;
        this.yMax = yMax;
        
        c = new Color(r.nextInt(256), r.nextInt(256), r.nextInt(256));
        
        xSpeed = 0;
        start();
    }
    
    /**
     * thread run method: move the ball periodically
     */
    public void run() {

        while (true) {
            
            try {
                sleep(PAUSE_TIME);
            }
            catch (InterruptedException e) {}
            
            // move the ball according to current velocity components
            y += ySpeed;
            x += xSpeed;
            
            // bounce off the floor!  include some damping so
            // we stop bouncing eventually
            if (y + BALL_SIZE > yMax) {
                y = yMax - BALL_SIZE;
                ySpeed = -ySpeed * 0.75;
            }
            
            // accelerate a bit left and right and some down to 
            // simulate gravity
            xSpeed += r.nextDouble()*0.25 - 0.125;
            ySpeed += .01;
        }
    }

    /**
     * paint this item on the given Graphics area
     * 
     * @param g Graphics area on which to draw
     */
    public void paint(Graphics g) {
    
        g.setColor(c);
        g.fillOval((int)x, (int)y, BALL_SIZE, BALL_SIZE);
    }
}

